
#include <vector>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "glframewindow.h"

// see "http://www.fltk.org/documentation.php/doc-1.1/toc.html" for
// more information

//////////////////////////////////////////////////////////////////
// define your gl window

#define MY_CIRCLE_LIST 1 

class MyGLFrameWindow : public GLFrameWindow {
public:
	MyGLFrameWindow();

	// overrideables from GLFrameWindow
	virtual void OnGLDraw();
	virtual void OnGLInit();
	virtual int handleForGLArea(int event);

	void Clear(){ points.clear(); };

private:
	// line drawing stuff
	struct point {
		point(int x, int y) : x(x), y(y){ }
		int x, y;
	};

	std::vector< point > points;
	int selectedPoint;
};

MyGLFrameWindow::MyGLFrameWindow() : GLFrameWindow(1024, 768, "linedraw"), selectedPoint(-1){
}

// draw stuff using opengl
// the opengl context has been created, set rendering options
void MyGLFrameWindow::OnGLInit(){
	// Turn on Gouraud shading
	glShadeModel(GL_SMOOTH);

	// Create a white background when clear colour buffers
	glClearColor(1.0, 1.0, 1.0, 0.0);

	// Initialize the cube display list
	glNewList(MY_CIRCLE_LIST, GL_COMPILE);
	glBegin(GL_LINE_STRIP);
	for(GLint i = 0; i <= 15; i++){
		GLfloat cosine = 5*cos(i*2*M_PI/15.0);
		GLfloat sine = 5*sin(i*2*M_PI/15.0);
		glVertex2f(cosine,sine);
	}
	glEnd();
	glEndList();

	// change the projective matrix 
	glMatrixMode(GL_PROJECTION); 
	// orthographic projection 
	glLoadIdentity(); 

	// set the viewport 
	glViewport(0.0,0.0, getGL().w(), getGL().h()); 
  
	glOrtho(0.0, getGL().w(), getGL().h(), 0.0, -0.1, 0.1); 

	// set modelview matix to identity 
	glMatrixMode( GL_MODELVIEW ); 
	glLoadIdentity();

	// setup font
	gl_font(FL_COURIER, 12);
};

void MyGLFrameWindow::OnGLDraw(){
	// Reset modelview matrix
	glLoadIdentity();

	// clear the window
	glClear( GL_COLOR_BUFFER_BIT );  

	// draw the line
	glColor3f( 0, 0, 0 );
	if ( points.size() > 1 ) {
		// draw a line using the OpenGL 
		glBegin(GL_LINE_STRIP);
		for (int i = 0; i < points.size(); i++) {
			glVertex2i( points[i].x, points[i].y );
		}
		glEnd();
	}
	
	char buf[20];
	glColor3f( 0, 0, 1 );
	for (int i = 0; i < points.size(); i++) {
		glPushMatrix();
			glTranslatef((float)points[i].x, (float)points[i].y, 0.0);
			glCallList(MY_CIRCLE_LIST);
		glPopMatrix();

		sprintf(buf, "p%d", i);
		// helpful fltk function
		gl_draw(buf, points[i].x+10, points[i].y+5);
	}

	// flush the buffer 
	glFlush();
};

// handle events sent to this window, or it's children
// NOTE: return 1 if you process an event, zero otherwise. 
int MyGLFrameWindow::handleForGLArea(int event){
	switch (event) {
	case FL_PUSH:
		if (Fl::event_state() & FL_BUTTON1) {
			// add a point to the line
			points.push_back( point(Fl::event_x(), Fl::event_y()) );

			getGL().redraw();
		}
		else if (Fl::event_state() & FL_BUTTON2){
			// select a point
			selectedPoint = -1;		

			for (int i = 0; i < points.size(); i++) {
				if ( (points[i].x-Fl::event_x())*(points[i].x-Fl::event_x()) + (points[i].y-Fl::event_y())*(points[i].y-Fl::event_y()) <= 36 ) {
					selectedPoint = i;
				}
			}	
		}
		return 1;

	case FL_DRAG:
		if (Fl::event_state() & FL_BUTTON2 && selectedPoint >= 0) {
			points[selectedPoint] = point(Fl::event_x(), Fl::event_y());
			getGL().redraw();
		}
		return 1;

	case FL_RELEASE:
		if (Fl::event_state() & FL_BUTTON2){
			selectedPoint = -1;
		}
		return 1;
	}
	return 0;
};

/////////////////////////////////////////////////////////////////
// your global variables

// main window
MyGLFrameWindow* glwindow = NULL;

void Clear(Fl_Widget *W, void* Data) {
	glwindow->Clear();
	glwindow->getGL().redraw();
}

void Quit(Fl_Widget *W, void* Data) {
	if(glwindow != NULL){
		delete glwindow;
		glwindow = NULL;
	}
}

Fl_Menu_Item mainMenuitems[] = {
  { "&File",       0, 0, 0, FL_SUBMENU },
    { "&Clear",    0, Clear, 0, FL_MENU_DIVIDER },
    { "&Quit",     0, Quit },
    { 0 },
  { 0 }
};

int main(int argc, char* argv[]){
	glwindow = new MyGLFrameWindow();

	// can also add FL_STENCIL, FL_ALPHA, FL_STEREO, FL_DEPTH
	// can test if a flag supported with getGL().can_do
	glwindow->getGL().mode(FL_RGB |FL_DOUBLE);

	// set the menu items
	glwindow->getMenu().copy(mainMenuitems);

	glwindow->show();

	return Fl::run();
}

